/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/


#include "stdafx.h"
#include <windows.h>
#include "devices.h"


#define _USE_IOCTL 1

#if _USE_IOCTL
	#include "UartMode.h"
#endif
/*
	Port[n]: 1  ; COM1
	         2  ; COM2
	         3  ; COM3
			 ....
	Mode[n]: 0  ; RS-232
	         1  ; RS-485 2W
	         2  ; RS-422
	         3  ; RS-485 4W
*/

int _tmain(int argc, _TCHAR* argv[])
{
	
	UCHAR	nPort;
	UCHAR	nMode;
	int		curMode;


	/*Port handle*/
	HANDLE	hCOM;

	/*Port information variables*/
	int		portNum=0;
	int		portStart=0;
	int		portEnd=0;
	
	/*Registry variables */
	HKEY	hKey;
	DWORD	ulRet;
	WCHAR   sPort[20];

	
	if(argc==3)
	{
		nPort=0x0+(UCHAR)_wtol(argv[1]);
		nMode=0x0+(UCHAR)_wtol(argv[2]);
		
#if _USE_IOCTL
		portMode=nMode;
		if((nPort>=1) && (nPort<=16) && (nMode<=3))
		{
			printf("Port=COM%d, Mode=%x \n",nPort,nMode);
			
			wsprintf(sPort,L"\\\\.\\COM%d",nPort);
			hCOM=CreateFile(sPort,  GENERIC_READ | GENERIC_WRITE,0,NULL,OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,NULL);

			if(hCOM != INVALID_HANDLE_VALUE) //open device success
			{
				SetPortVal(hCOM);
			}
			GetPortVal(hCOM);
			CloseHandle(hCOM);
		
		}
		else
		{
			printf("Invalid parameter! Please try again.\n");
			Usage();
			exit(0);
		}
	
		/*Set Registry Value*/
		ulRet = RegOpenKeyEx( HKEY_LOCAL_MACHINE, TEXT("SOFTWARE\\MOXA\\COM"), NULL, KEY_ALL_ACCESS, &hKey);
		if ( ulRet == ERROR_SUCCESS )
		{
			wsprintf( sPort, L"COM%d", nPort);
			ulRet = RegSetValueEx( hKey, sPort, NULL, REG_DWORD, (LPBYTE)&portMode, sizeof(DWORD) );
			if ( ulRet != ERROR_SUCCESS ) 
			{	
				printf("Set registry value error!\n");	
			}
		}
#else
		/* Set Mode Configuration */
		if(setUrMode(nPort,nMode))
		{
			printf("set OK\n");
		}

		/* Get Mode Configuration */
		curMode=getUrMode(nPort);
		if(curMode==(-1))
		{
			printf("error");
		}
		else
		{
			printf("get OK\n");
			printf("port:%d, get current mode %d\n",nPort,curMode);
		}
	
	
#endif
	}
	else
	{
		printf("Invalid parameter! Please try again.\n");
		Usage();
	}


	return 0;
}

	
	
#if _USE_IOCTL

BOOL SetPortVal(HANDLE hndFile)
{
	IoctlCode = IOCTL_SET_MODE;
	IoctlResult = DeviceIoControl(
						hndFile,                // Handle to device
						IoctlCode,              // IO Control code for Write
						&portMode,				// Buffer to driver.  Holds port & data.
						sizeof(portMode),       // Length of buffer in bytes.
						NULL,                   // Buffer from driver.   Not used.
						0,                      // Length of buffer in bytes.
						&ReturnedLength,        // Bytes placed in outbuf.  Should be 0.
						NULL                    // NULL means wait till I/O completes.
						);

	if (!IoctlResult)
	{
		printf("Ioctl failed with code %ld\n", GetLastError() );
	}
	return IoctlResult;
}

BOOL GetPortVal(HANDLE hndFile)
{
	IoctlCode = IOCTL_GET_MODE;
	
	IoctlResult = DeviceIoControl(
						hndFile,                // Handle to device
						IoctlCode,              // IO Control code for Write
						NULL,					// Buffer to driver.  Holds port & data. Not used.
						0,						// Length of buffer in bytes. Not used.
						&portMode,				// Buffer from driver.   
						sizeof(portMode),       // Length of buffer in bytes.
						&ReturnedLength,        // Bytes placed in outbuf.  Should be 0.
						NULL                    // NULL means wait till I/O completes.
						);
		
	if (!IoctlResult)
	{
		printf("Ioctl failed with code %ld\n", GetLastError() );
	}
	
	return IoctlResult;
}
#endif
void Usage()
{
	printf("\n\n");
	printf("Usage: \n");
	printf("\tSetInterface  [Port] [Mode]    ; Port=1~16  Mode=0,1,2,3\n");
	
	printf("\t Mode 0: RS232 \n");
	printf("\t Mode 1: RS485-2W \n");
	printf("\t Mode 2: RS422 \n");
	printf("\t Mode 3: RS485-4W \n");
	printf("\n");

	printf("e.g.\tSetInterface 3 2     ; Change the COM3: to RS422\n");
	printf("    \tSetInterface 4 3     ; Change the COM4: to RS485-4W\n");
}